/***************************************************************************
 *  Copyright  2008  Marcus Bauer
 *  License    GPLv2
 ****************************************************************************/

#include <glib.h>
#include <glib/gprintf.h>
#include <gtk/gtk.h>
#include <string.h>
#include <stdlib.h>
#include <gdk/gdk.h>
#include <gconf/gconf.h>
#include <math.h>
#include <time.h>

#include "globals.h"
#include "map_management.h"
#include "converter.h"
#include "gps_functions.h"
#include "support.h"

gboolean
cb_gps_timer()
{
	int pixel_x, pixel_y, x,y;
	static float lat, lon;
	static float lat_tmp, lon_tmp;
	float	trip_delta=0;
	double time_sec;
	static double trip_time_accumulated;
	struct tm *tm_p;
	int trip_hours, trip_minutes, trip_seconds;
	static gboolean trip_counter_got_stopped = FALSE;

//its global now	static float trip = 0.0;
	GdkColor color;
	GdkGC *gc, *gc_2;
	
	gc   = gdk_gc_new(pixmap);
	gc_2 = gdk_gc_new(pixmap);
	color.green = 0;
	color.blue = 0;
	color.red = 60000;
	gdk_gc_set_rgb_fg_color(gc, &color);

	color.green = 30000;
	color.blue = 30000;
	color.red = 65000;
	gdk_gc_set_rgb_fg_color(gc_2, &color);
	
	printf("timer called\n");

	get_gps();
	
	//lat = deg2rad(51.52468);
	//lon = deg2rad(-0.13762);
	if(gpsdata)
	{
		trackpoint_t *tp = g_new0(trackpoint_t,1);
		static int counter = 0;
		
		//TODO: keep last point in mem and overpaint it black?
		lat_tmp = lat;
		lon_tmp = lon;
		lat = deg2rad(gpsdata->fix.latitude);
		lon = deg2rad(gpsdata->fix.longitude);
		
		printf("%s \n", __PRETTY_FUNCTION__);
		
		
		// pixel_x,y, offsets
		pixel_x = lon2pixel(global_zoom, lon);
		pixel_y = lat2pixel(global_zoom, lat);
		
		x = pixel_x - global_x;
		y = pixel_y - global_y;
		

		
		gdk_draw_arc (
			pixmap,
			//map_drawable->style->black_gc,
			gc,
			TRUE,			//filled
			x-4, y-4,		// x,y
			8,8,			// width, height
			0,23040);		// start-end angle 64th, from 3h, anti clockwise
		
		gtk_widget_queue_draw_area (
			map_drawable, 
			x-4, y-4,
			8,8);
			
		
		if(global_autocenter)
		{
			if(    (x < (global_drawingarea_width /2 - global_drawingarea_width /8) ||
				x > (global_drawingarea_width /2 + global_drawingarea_width /8) ||
				y < (global_drawingarea_height /2 - global_drawingarea_height /8) ||
				y > (global_drawingarea_height /2 + global_drawingarea_height /8) ) &&
			
				/* vaild data? */
				isnan(gpsdata->fix.latitude) ==0 &&
				isnan(gpsdata->fix.longitude)==0 &&
				gpsdata->fix.latitude  !=0 &&
				gpsdata->fix.longitude !=0
				)
			{
				set_mapcenter(gpsdata->fix.latitude, gpsdata->fix.longitude, global_zoom);
			}
		}
		
		if(trip_counter_on){
		if(	lat!=0 && lon!=0 && lat_tmp!=0 && lon_tmp!=0 && 
			isnan(lat)==0 && isnan(lon)==0 && 
			isnan(lat_tmp)==0 && isnan(lon_tmp)==0)
		{

			
			trip_delta = 6731.0 *  acos(sin(lat) * sin(lat_tmp) + 
							cos(lat) * cos(lat_tmp) * cos(lon_tmp-lon) );
			if(isnan(trip_delta))
			{
				printf("WTF??? %f %f %f %f %f \n",lat,lon,lat_tmp,lon_tmp,trip_delta);
			}
			else
			{
				trip_distance += trip_delta;
				printf("TRIP %f %f %f %f %f \n",lat,lon,lat_tmp,lon_tmp,trip_delta);
				
				if(trip_distance > 0.005)
				{
					// TODO: fixed # of tp maybe?
					counter++;
					if(counter % 5 == 0)
					{
						tp->lat = lat;
						tp->lon = lon;
						trackpoint_list = g_slist_append(trackpoint_list, tp);
					}	
				}
			}
			printf("trip: %f \n", trip_distance);
		}
		
		if(isnan(gpsdata->fix.time)==0)
			time_sec = gpsdata->fix.time;
		if(isnan(gpsdata->fix.speed)==0 && gpsdata->fix.speed > trip_maxspeed)
			trip_maxspeed = gpsdata->fix.speed;
		
		// restart/ reset handling
		if(trip_time == 0) trip_time_accumulated = 0;
		if(trip_counter_got_stopped)
		{
			printf("counter had been stopped \n");
			trip_counter_got_stopped = FALSE;
			trip_time_accumulated = trip_time;
			trip_starttime = 0;
		}
		
		if(trip_starttime == 0 && time_sec >1100000000 && time_sec < 1419120000)
		{
			printf("******************************setting trip starttime\n");
			trip_starttime = time_sec;
			
			printf("tripSTART: %.0f sec \n", time_sec);
		}
		if(trip_starttime !=0 && time_sec !=0 && time_sec >1100000000 && time_sec < 1419120000)
		{
			trip_time = time_sec - trip_starttime + trip_time_accumulated;
			
			printf("timeSEC: %.0f sec \n", time_sec);
		}
		
		// hours /3600, min (sec%3600)/60, second (sec%60)
		trip_hours   = trip_time / 3600;
		trip_minutes = ((int)trip_time%3600)/60;
		trip_seconds = (int)trip_time % 60;

		// <10 min,sec;; BEI RESET aUF NULL, global_tripmeter_on =TRUE/FALSE
		printf("TRIPTIME: %.0f - %d:%2d:%2d - %.1f max: %.1f\n", 
			trip_time,trip_hours,trip_minutes,trip_seconds,trip_distance*3600/trip_time,
			trip_maxspeed*3.6);
		
		//avg,max,hohenmeter
		}
		else
		{
			printf("trip counter halted\n");
			trip_counter_got_stopped = TRUE;
			lat_tmp = lon_tmp = 0;
		}
		set_label();

	}
	else printf("no gpsdata for timer\n");
	return TRUE; // FALSE-> end timer
}

GSList *
get_gconf_repo_list()
{
	GSList	*repo_list;
	
	GSList	*list;
	GError **error = NULL;
	
	
	repo_list = gconf_client_get_list(	global_gconfclient, 
						GCONF"/repos",
						GCONF_VALUE_STRING,
						error);
	
	if (repo_list == NULL)
	{
		repo_t *repo = g_new0(repo_t, 1);
		printf("REPOLIST == NULL\n");
		repo->name = g_strdup("OSM");
		repo->uri  = g_strdup("http://tile.openstreetmap.org/%d/%d/%d.png");
		repo->dir  = g_strdup("/tmp/Maps/OSM");
		repo->inverted_zoom = 0;
		global_repo_list = g_slist_append(global_repo_list, repo);
	}
	
	/* deserialize GSList of strings into GSList of struct */
	for(list = repo_list; list != NULL; list = list->next)
	{
		gchar **array;
		gchar *str = list->data;
		repo_t *repo = g_new0(repo_t, 1);
		
		array = g_strsplit(str,"|",4);	//TODO free array gchar **arr1;
		//TODO check array length == 4
		repo->name = g_strdup(array[0]);
		repo->uri  = g_strdup(array[1]);
		repo->dir  = g_strdup(array[2]);
		repo->inverted_zoom = (atoi(g_strdup(array[3])) == 1) ? TRUE : FALSE;
		
		global_repo_list = g_slist_append(global_repo_list, repo);

		printf("GCONF: \n -- name: %s \n -- uri: %s \n -- dir: %s \n",
			repo->name, repo->uri, repo->dir);
	}
	
	
	return global_repo_list;	//TODO do we need this return ever? ->remove?
					//besser: bool wenn NULL, dann default setzen
}

void
repo_config_to_gconf()
{
	
	GSList	*list;
	GSList	*gconf_list = NULL;
	GError **error = NULL;
	gboolean success = FALSE;
	
	

	
	/* serialize GSList of struct into GSList of strings */
	for(list = global_repo_list; list != NULL; list = list->next)
	{
		repo_t *repo;
		gchar gconf_str[1024];
		
		repo = list->data;
		
		
		g_sprintf(	gconf_str, 
				"%s|%s|%s|%i",
				repo->name, repo->uri, repo->dir, repo->inverted_zoom);
		

		
		gconf_list = g_slist_append(gconf_list, g_strdup(gconf_str));

		printf("GCONFSAVE: \n -- name: %s \n -- uri: %s \n -- dir: %s \n -- zoom: %i \n\n %s \n",
			repo->name, repo->uri, repo->dir, repo->inverted_zoom, gconf_str);
	}
	
	success = gconf_client_set_list(	global_gconfclient, 
						GCONF"/repos",
						GCONF_VALUE_STRING,
						gconf_list,
						error);
	
	printf("*** %s(): %i \n",__PRETTY_FUNCTION__, success);

}
//inverted zoom
// für tile und mapmanagement nutzen

// track, gps, trip -- website, 
// GCT, urssaf, steuernr., CA.

//repo config apply changes: update global repo,, gconf
//repo config new
// default values
//gconf save
//doc gconf, lists, struct, combobox, serialize/deserialize
//apply button, label, timer

void
repo_config_populate_dialog()
{
	GtkWidget	*entry_uri;
	GtkWidget	*entry_dir;
	GtkWidget	*togglebutton;
	repo_t		*repo;
	
	entry_uri = lookup_widget(window1, "entry1");
	entry_dir = lookup_widget(window1, "entry2");
	togglebutton = lookup_widget(window1, "checkbutton1");
	//uri, dir,,, inversed
	// lookup,, set entry
	repo = global_repo_curr->data;
	gtk_entry_set_text(entry_uri, repo->uri);
	gtk_entry_set_text(entry_dir, repo->dir);
	gtk_toggle_button_set_active(togglebutton, repo->inverted_zoom);
}

void
repo_config_set_current()
{
	// herausfinden #pos von reponame -> vergleich g_strrstr, strlen
	// setzen	
	GSList		*list;
	const gchar	*reponame;
	
	for(list = global_repo_list; list != NULL; list = list->next)
	{
		repo_t	*repo;
		
		repo	 = list->data;
		reponame = g_strdup(repo->name);
		
		if(	g_strrstr(reponame,global_repo_curr_name) != NULL &&
			strlen(reponame) == strlen(global_repo_curr_name)	
		)
			global_repo_curr = list;
	}
}


void
repo_config_create_dropdown()
{
	GtkWidget	*combobox;
	GSList		*list;
	int 		i = 0;
	int		j = 0;
	const gchar	*reponame;
	
	combobox = lookup_widget(window1, "combobox1");

	for(list = global_repo_list; list != NULL; list = list->next)
	{
		repo_t	*repo;
		
		repo = list->data;
		reponame = g_strdup(repo->name);
		gtk_combo_box_append_text (combobox, g_strdup(repo->name));
		
		if(	g_strrstr(reponame,global_repo_curr_name) != NULL &&
			strlen(reponame) == strlen(global_repo_curr_name)	
		)
		{
			j = i;
			global_repo_curr = list;
		}
		i++;
	}
	global_repo_cnt = i;
	gtk_combo_box_set_active(GTK_COMBO_BOX(combobox), j);
}


void
init0()
{
	GError	**err = NULL;

	g_type_init();
	global_gconfclient	= gconf_client_get_default();
	global_repo_curr_name	= gconf_client_get_string(global_gconfclient, GCONF"/repo_name",err);


	if(global_repo_curr_name == NULL)
	{
		printf("gconf repo_name not set\n");
		global_repo_curr_name = g_strdup("OSM");
	}
	get_gconf_repo_list();	
	repo_config_set_current();
}

void
init()
{
	gint timer;
	gpointer data = NULL;
	float lat, lon;
	
	
	GError	**err = NULL;
	const gchar *nick, *pass;
	GtkWidget *nick_entry, *pass_entry;

	nick_entry  = lookup_widget(window1, "entry7");
	pass_entry  = lookup_widget(window1, "entry8");

	global_gconfclient	= gconf_client_get_default();
	nick			= gconf_client_get_string(global_gconfclient, GCONF"/nick",err);
	pass			= gconf_client_get_string(global_gconfclient, GCONF"/pass",err);
	
	gtk_entry_set_text(nick_entry, nick);
	gtk_entry_set_text(pass_entry, pass);
	
	
	
	global_myposition.lat = 0;
	global_myposition.lon = 0;
	global_zoom = 3;
	lat = deg2rad(66.3);
	lon = deg2rad(-23.3);

	//TODO: for faster startup maybe we can just set globals and have d-c-e() doint it?
	fill_tiles_latlon(lat, lon, global_zoom);

	repo_config_create_dropdown();
	
	timer = g_timeout_add (1000,cb_gps_timer,data);
}
